<?php

require_once _PS_MODULE_DIR_ . 'prestashopgatewaypayphone/lib/api/PayphoneButtonPpb.php';
require_once _PS_MODULE_DIR_ . 'prestashopgatewaypayphone/lib/models/request/ConfirmSaleRequestModelPpb.php';
require_once _PS_MODULE_DIR_ . 'prestashopgatewaypayphone/lib/models/response/SaleGetResponseModelPpb.php';
require_once _PS_MODULE_DIR_ . 'prestashopgatewaypayphone/lib/common/PayPhoneWebExceptionPpb.php';

class PrestashopGatewayPayphoneValidationModuleFrontController extends ModuleFrontController
{
    /**
     * @var PaymentModule
     */
    public $module;

    /**
     * {@inheritdoc}
     */
    public function postProcess()
    {
        if (false === $this->checkIfContextIsValid() || false === $this->checkIfPaymentOptionIsAvailable()) {
            Tools::redirect(
                $this->context->link->getPageLink(
                    'order',
                    true,
                    (int) $this->context->language->id,
                    [
                        'step' => 1,
                    ]
                )
            );
        }

        $customer = new Customer($this->context->cart->id_customer);

        if (false === Validate::isLoadedObject($customer)) {
            Tools::redirect(
                $this->context->link->getPageLink(
                    'order',
                    true,
                    (int) $this->context->language->id,
                    [
                        'step' => 1,
                    ]
                )
            );
        }

        $this->module->validateOrder(
            (int) $this->context->cart->id,
            (int) $this->getOrderState(),
            (float) $this->context->cart->getOrderTotal(true, Cart::BOTH),
            'Payphone PaymentBox',
            null,
            [
                'transaction_id' => Tools::passwdGen(), // Should be retrieved from your Payment response
            ],
            (int) $this->context->currency->id,
            false,
            $customer->secure_key
        );

        $this->updateReferenceOrder($this->context->cart->id);

        Tools::redirect(
            $this->context->link->getPageLink(
                'order-confirmation',
                true,
                (int) $this->context->language->id,
                [
                    'id_cart' => (int) $this->context->cart->id,
                    'id_module' => (int) $this->module->id,
                    'id_order' => (int) $this->module->currentOrder,
                    'key' => $customer->secure_key,
                ]
            )
        );
    }

    /**
     * Check if the context is valid
     *
     * @return bool
     */
    private function checkIfContextIsValid()
    {
        return true === Validate::isLoadedObject($this->context->cart)
            && true === Validate::isUnsignedInt($this->context->cart->id_customer)
            && true === Validate::isUnsignedInt($this->context->cart->id_address_delivery)
            && true === Validate::isUnsignedInt($this->context->cart->id_address_invoice);
    }

    /**
     * Check that this payment option is still available in case the customer changed
     * his address just before the end of the checkout process
     *
     * @return bool
     */
    private function checkIfPaymentOptionIsAvailable()
    {
        $modules = Module::getPaymentModules();

        if (empty($modules)) {
            return false;
        }

        foreach ($modules as $module) {
            if (isset($module['name']) && $this->module->name === $module['name']) {
                return true;
            }
        }

        return false;
    }

    /**
     * Get OrderState identifier
     *
     * @return int
     */
    private function getOrderState()
    {

        $orderStateId = (int) Configuration::get('PS_PAYPHONE_PPB_REJECTED');

        // Get client_transaction_id for confirm
        $sql = new DbQuery();
        $sql->select('*');
        $sql->from('payphone_ppb', 'pt');
        $sql->where('pt.client_user_id = ' . $this->context->cart->id);
        $sql->orderBy('pt.id DESC');
        $transactions = Db::getInstance()->executeS($sql);

        if ($transactions) {
            $clientTransactionId = $transactions[0]['client_transaction_id'];
            $amount = $transactions[0]['amount'];
        }

        if (!$clientTransactionId || !$amount) {
            PrestaShopLogger::addLog("PayPhone validation controller error = clienteTransactionId null", 2);
            return $orderStateId;
        }

        $config = ConfigurationManagerPpb::Instance();
        $config->Token = Configuration::get('PAYPHONE_TOKEN_PPB');
        $pb = new PayphoneButtonPpb();

        try {
            $data = $pb->GetSaleByClientId($clientTransactionId);
            if (is_array($data)) {
                $numberOfElements = count($data);
                $data = $data[$numberOfElements - 1];
                $amount1 = $amount . "";
                $amount2 = $data->amount . "";

                $updateData = [
                    'transaction_id' => pSQL($data->transactionId),
                    'currency' => pSQL($data->currency),
                    'store_name' => pSQL($data->storeName),
                    'client_pay_names' => ($data->optionalParameter4) ? pSQL($data->optionalParameter4) : null,
                    'client_pay_phone_number' => pSQL($data->phoneNumber),
                    'deferred' => $data->deferred,
                ];
                if ($data->deferred)
                    $updateData['deferred_message'] = pSQL($data->deferredMessage);
                $whereClause = ' `client_user_id` = ' . $this->context->cart->id;

                if ($amount1 != $amount2) {
                    $errorMessage = $this->l('The transaction value does not match the order value.');
                    $updateData['message'] = pSQL($errorMessage);
                    $updateData['status_code'] = 2;
                    PrestaShopLogger::addLog("PayPhone validation controller error = Error en la data", 2);
                    Db::getInstance()->update('payphone_ppb', $updateData, $whereClause);
                    return $orderStateId;
                } else {
                    $updateData['message'] = ($data->message) ? pSQL($data->message) : null;
                    $updateData['status_code'] = (int) $data->statusCode;
                }

                $transactionId = $data->transactionId;
                $clientTransactionId = $data->clientTransactionId;
                Db::getInstance()->update('payphone_ppb', $updateData, $whereClause);

                if ($data->statusCode == 2)
                    return $orderStateId;
                $status = $this->paymentResponse($transactionId);
                return $status;

            } else {
                if ($data->error->message == 'Transacción no encontrada') {
                    PrestaShopLogger::addLog("PayPhone GetSaleByClientId error = 'Transacción no encontrada'", 2);
                } else {
                    PrestaShopLogger::addLog("PayPhone GetSaleByClientId error =" . $data->error->errors[0], 2);
                }
                return $orderStateId;
            }
        } catch (PayPhoneWebExceptionPpb $e) {
            PrestaShopLogger::addLog("PayPhone GetSaleByClientId error =" . $e->ErrorList[0]->message, 2);
            return $orderStateId;
        }

        return $orderStateId;
    }


    /**
     * Gets the result of the response of the payment button
     * Call to the confrim to record the transaction
     */
    private function paymentResponse($transactionId)
    {

        $config = ConfigurationManagerPpb::Instance();
        $config->Token = Configuration::get('PAYPHONE_TOKEN_PPB');
        $pb = new PayphoneButtonPpb();
        try {
            $result = $pb->Confirm($transactionId);
            $data = $result;
            if ($data->statusCode == 3) {
                $updateData = [
                    'status_code' => (int) $data->statusCode,
                    'message' => ($data->message) ? pSQL($data->message) : null,
                    'card_bran' => pSQL($data->cardBrand),
                    'authorization_code' => pSQL($data->authorizationCode),
                ];
                $whereClause = ' `client_user_id` = ' . $this->context->cart->id;
                Db::getInstance()->update('payphone_ppb', $updateData, $whereClause);
                return (int) Configuration::get('PS_PAYPHONE_PPB_APPROVED');
            } else {
                return (int) Configuration::get('PS_PAYPHONE_PPB_REJECTED');
            }
        } catch (PayPhoneWebExceptionPpb $e) {
            PrestaShopLogger::addLog("PayPhone GetSaleByClientId error =" . $e->ErrorList[0]->message, 2);
            return (int) Configuration::get('PS_PAYPHONE_PPB_REJECTED');
        }
    }

    /**
     * Update the order reference with the reference generated for payment with payphone
     */
    private function updateReferenceOrder($cartId)
    {
        $reference = $this->context->cookie->payphone_reference;
        if ($reference && $cartId) {
            Db::getInstance()->update(
                'orders',
                array('reference' => pSQL($reference)),
                'id_cart = ' . (int) $cartId
            );
            unset($this->context->cookie->payphone_reference);
        }
    }
}