<?php

use PrestaShop\PrestaShop\Core\Payment\PaymentOption;

if (!defined('_PS_VERSION_')) {
    exit;
}

require_once _PS_MODULE_DIR_ . 'prestashopgatewaypayphone/lib/api/PayphoneButtonPpb.php';
require_once _PS_MODULE_DIR_ . 'prestashopgatewaypayphone/lib/Configuration.php';
require_once _PS_MODULE_DIR_ . 'prestashopgatewaypayphone/lib/common/Constants.php';

class PrestashopGatewayPayphone extends PaymentModule
{
    const MODULE_ADMIN_CONTROLLER = 'AdminConfigurePrestashopGatewayPayphone';
    const HOOKS = [
        'actionObjectShopAddAfter',
        'paymentOptions',
        'displayAdminOrderMainBottom',
        'displayOrderDetail',
        'displayPaymentByBinaries',
        'displayPaymentReturn',
    ];

    protected $_html = '';
    protected $_postErrors = array();
    public $token;
    public $store;
    public $test;
    private $config = array('PAYPHONE_TOKEN_PPB', 'PAYPHONE_STORE_PPB');
    private $config_vars = array(
        'PAYPHONE_TOKEN_PPB' => 'Token is required.',
    );
    private $payphone_pending = array(
        'PS_PAYPHONE_PPB_PENDING' => array(
            'es' => 'Pendiente de pago con Payphone PaymentBox',
            'en' => 'Pending Payphone PaymentBox payment'
        )
    );
    private $payphone_green = array(
        'PS_PAYPHONE_PPB_APPROVED' => array(
            'es' => 'Pago con Payphone PaymentBox aprobado',
            'en' => 'Accepted Payphone PaymentBox payment'
        ),
    );
    private $payphone_red = array(
        'PS_PAYPHONE_PPB_REJECTED' => array(
            'es' => 'Pago con Payphone PaymentBox cancelado',
            'en' => 'Canceled or error Payphone PaymentBox payment'
        ),
    );

    public function __construct()
    {
        $this->name = 'prestashopgatewaypayphone';
        $this->tab = 'payments_gateways';
        $this->version = '1.0.0';
        $this->author = 'Payphone Developer';
        $this->bootstrap = true;
        $this->currencies = true;
        $this->currencies_mode = 'checkbox';

        $config_values = Configuration::getMultiple($this->config);

        if (!empty($config_values['PAYPHONE_TOKEN_PPB']))
            $this->token = $config_values['PAYPHONE_TOKEN_PPB'];
        if (!empty($config_values['PAYPHONE_STORE_PPB']))
            $this->store = $config_values['PAYPHONE_STORE_PPB'];

        $this->ps_versions_compliancy = [
            'min' => '1.7',
            'max' => _PS_VERSION_,
        ];
        $this->controllers = [
            'validation',
        ];

        parent::__construct();

        $this->displayName = $this->l('Payphone PaymentBox');
        $this->description = $this->l('Accept payments in your online store with Visa, Mastercard or Payphone balance through our plugin.');
        $this->confirmUninstall = $this->l('Are you sure about removing?');

        if (!isset($this->token))
            $this->warning = $this->l('You must configure token parameter for this module');
        if (!count(Currency::checkPaymentCurrencies($this->id)))
            $this->warning = $this->l('No currency has been set for this module.');
    }

    /**
     * @return bool
     */
    public function install()
    {
        return (bool) parent::install()
            && (bool) $this->registerHook(static::HOOKS)
            && $this->installTabs()
            && $this->_installDb()
            && $this->_createOrderStates();
    }

    /**
     * @return bool
     */
    public function uninstall()
    {
        return (bool) parent::uninstall()
            && $this->deleteOrderState()
            && $this->uninstallTabs();
    }

    /**
     * Validates configuration variables
     */
    protected function _postValidation()
    {

        if (Tools::isSubmit('btnSubmit')) {
            foreach ($this->config_vars as $key => $value) {
                if (!Tools::getValue($key))
                    $this->_postErrors[] = $this->l($value);
            }
        }
    }

    /**
     * Saves the values of the configuration variables
     */
    protected function _postProcess()
    {

        if (Tools::isSubmit('btnSubmit')) {
            foreach ($this->config as $key) {
                Configuration::updateValue($key, Tools::getValue($key));
            }
        }
        $this->_html .= $this->displayConfirmation($this->l('Settings updated'));
    }

    /**
     * Returns the value of the configuration variables
     * @return array
     */
    public function getConfigFieldsValues()
    {

        $fields = array();
        foreach ($this->config as $key) {
            $fields[$key] = Tools::getValue($key, Configuration::get($key));
        }
        return $fields;
    }

    /**
     * Set configuration form fields using the forms API
     * @return type
     */
    public function renderForm()
    {

        $fields_form = array(
            'form' => array(
                'legend' => array(
                    'title' => $this->l('Variables'),
                    'icon' => 'icon-envelope'
                ),
                'input' => array(
                    array(
                        'type' => 'textarea',
                        'label' => $this->l('Token'),
                        'name' => 'PAYPHONE_TOKEN_PPB',
                        'required' => true
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->l('Store id'),
                        'name' => 'PAYPHONE_STORE_PPB',
                        'desc' => $this->l('Store id in payphone')
                    ),
                ),
                'submit' => array(
                    'title' => $this->trans('Save', array(), 'Admin.Actions'),
                    'class' => 'btn btn-default pull-right'
                )
            ),
        );
        $helper = new HelperForm();
        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $lang = new Language((int) Configuration::get('PS_LANG_DEFAULT'));
        $helper->default_form_language = $lang->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') ? Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') : 0;
        $this->fields_form = array();
        $helper->id = (int) Tools::getValue('id_carrier');
        $helper->identifier = $this->identifier;
        $helper->submit_action = 'btnSubmit';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false) . '&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->tpl_vars = array(
            'fields_value' => $this->getConfigFieldsValues(),
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id
        );
        return $helper->generateForm(array($fields_form));
    }



    public function getContent()
    {
        if (Tools::isSubmit('btnSubmit')) {
            $this->_postValidation();
            if (!count($this->_postErrors))
                $this->_postProcess();
            else
                foreach ($this->_postErrors as $err)
                    $this->_html .= $this->displayError($err);
        } else
            $this->_html .= '<br />';
        $this->_html .= '<p><b>' . $this->l('Prestashop response URL') . ":</b> " . $this->context->link->getModuleLink('prestashopgatewaypayphone', 'validation', array(), Configuration::get('PS_SSL_ENABLED')) . "</p>";
        $this->_html .= $this->renderForm();

        return $this->_html;

    }


    /**
     * This hook called after a new Shop is created
     *
     * @param array $params
     */
    public function hookActionObjectShopAddAfter(array $params)
    {
        if (empty($params['object'])) {
            return;
        }

        /** @var Shop $shop */
        $shop = $params['object'];

        if (false === Validate::isLoadedObject($shop)) {
            return;
        }

        $this->addCheckboxCarrierRestrictionsForModule([(int) $shop->id]);
        $this->addCheckboxCountryRestrictionsForModule([(int) $shop->id]);

        if ($this->currencies_mode === 'checkbox') {
            $this->addCheckboxCurrencyRestrictionsForModule([(int) $shop->id]);
        } elseif ($this->currencies_mode === 'radio') {
            $this->addRadioCurrencyRestrictionsForModule([(int) $shop->id]);
        }
    }

    /**
     * @param array $params
     *
     * @return array Should always return an array
     */
    public function hookPaymentOptions(array $params)
    {

        if (!$this->active) {
            return;
        }

        /** @var Cart $cart */
        $cart = $params['cart'];

        if (false === Validate::isLoadedObject($cart) || false === $this->checkCurrency($cart)) {
            return [];
        }

        $this->context->smarty->assign(
            array(
                'module_dir' => _MODULE_DIR_
            )
        );

        $paymentOptions = [];
        $paymentOptions[] = $this->getBinaryPaymentOption();

        return $paymentOptions;
    }

    /**
     * This hook is used to display additional information on BO Order View, under Payment block
     *
     * @since PrestaShop 1.7.7 This hook replace displayAdminOrderLeft on migrated BO Order View
     *
     * @param array $params
     *
     * @return string
     */
    public function hookDisplayAdminOrderMainBottom(array $params)
    {
        if (empty($params['id_order'])) {
            return '';
        }

        $order = new Order((int) $params['id_order']);

        if (false === Validate::isLoadedObject($order) || $order->module !== $this->name) {
            return '';
        }

        $css_uri = $this->getPathUri() . 'views/css/ps-ppb.css';
        $cartId = $order->id_cart;
        $orderReference = $order->reference;
        $hook_display_option = 3;

        $data_to_context = $this->getPayphoneOrderData($cartId, $css_uri, $orderReference, $hook_display_option);
        $data_to_context = json_decode($data_to_context, true);

        $this->context->smarty->assign($data_to_context);

        return $this->context->smarty->fetch('module:prestashopgatewaypayphone/views/templates/hook/payphone-order-template.tpl');
    }


    /**
     * This hook is used to display additional information on FO 
     *
     * @param array $params
     *
     * @return string
     */
    public function hookDisplayOrderDetail(array $params)
    {

        if (empty($params['order'])) {
            return '';
        }

        /** @var Order $order */
        $order = $params['order'];

        if (false === Validate::isLoadedObject($order) || $order->module !== $this->name) {
            return '';
        }

        $css_uri = $this->getPathUri() . 'views/css/ps-ppb.css';
        $cartId = $order->id_cart;
        $orderReference = $order->reference;
        $hook_display_option = 2;

        $data_to_context = $this->getPayphoneOrderData($cartId, $css_uri, $orderReference, $hook_display_option);
        $data_to_context = json_decode($data_to_context, true);

        $this->context->smarty->assign($data_to_context);

        return $this->context->smarty->fetch('module:prestashopgatewaypayphone/views/templates/hook/payphone-order-template.tpl');
    }

    /**
     * This hook displays form generated by binaries during the checkout
     *
     * @param array $params
     *
     * @return string
     */
    public function hookDisplayPaymentByBinaries(array $params)
    {
        /** @var Cart $cart */
        $cart = $params['cart'];

        if (false === Validate::isLoadedObject($cart) || false === $this->checkCurrency($cart)) {
            return '';
        }


        $data = new PrepareSaleRequestModelPpb();
        $tk = Configuration::get('PAYPHONE_TOKEN_PPB');

        try {

            $cart_summary = json_decode(json_encode($this->context->cart->getSummaryDetails()), true);
            $products = Context::getContext()->cart->getProducts();
            $emailCart = Context::getContext()->customer->email;
            $country = $this->context->country->iso_code;
            $cart = $this->context->cart;

            if (false === $this->checkCurrency($cart)) {
                Tools::redirect('index.php?controller=order');
            }

            $currency = $this->context->currency;

            /*######     DATOS DE MONTOS    #########*/
            $productSinIva = $productConIva = $envioCon = $envioSin = $descuentoCon = $descuentoSin = $envoltorioCon = $envoltorioSin = 0;
            $products = $cart_summary['products'];
            foreach ($products as $key => $product) {
                if ($product["total"] == $product["total_wt"]) {
                    $productSinIva += round($product["total"], 2);
                } else {
                    $productConIva += round($product["total"], 2);
                }
            }


            if ($cart_summary['total_shipping'] == $cart_summary['total_shipping_tax_exc']) {
                $envioSin = round($cart_summary['total_shipping_tax_exc'], 2);
            } else {
                $envioCon = round($cart_summary['total_shipping_tax_exc'], 2);
            }

            if ($cart_summary['total_discounts'] == $cart_summary['total_discounts_tax_exc']) {
                $descuentoSin = round($cart_summary['total_discounts_tax_exc'], 2);
            } else {
                $descuentoCon = round($cart_summary['total_discounts_tax_exc'], 2);
            }

            if ($cart_summary['total_wrapping'] == $cart_summary['total_wrapping_tax_exc']) {
                $envoltorioSin = round($cart_summary['total_wrapping_tax_exc'], 2);
            } else {
                $envoltorioCon = round($cart_summary['total_wrapping_tax_exc'], 2);
            }

            $sub_total = round($cart_summary['total_products_wt'], 2);
            $shipping = round($cart_summary['total_shipping'], 2);

            $amount_with_tax = round(($productConIva + $envioCon + $envoltorioCon - $descuentoCon), 2);
            $amount_without_tax = round(($productSinIva + $envioSin + $envoltorioSin - $descuentoSin), 2);
            $amount = round($cart_summary['total_price'], 2);
            $tax = round($cart_summary['total_tax'], 2);

            $datosFactura = $cart_summary['invoice'];
            $lineItems = [];
            $billTo = [
                "address1" => $datosFactura['address1'],
                "address2" => $datosFactura['address2'],
                "country" => $country,
                "state" => $datosFactura['country'],
                "locality" => $datosFactura['city'],
                "firstName" => $datosFactura['firstname'],
                "lastName" => $datosFactura['lastname'],
                "phoneNumber" => $datosFactura['phone'],
                "email" => $emailCart,
                "postalCode" => $datosFactura['postcode'],
                "customerId" => $datosFactura['id_customer']
            ];

            foreach ($products as $product) {
                $productos = [
                    "productName" => substr(trim(strip_tags($product['name'])), 0, 50),
                    "unitPrice" => round(round($product['price'], 2) * 100, 2),
                    "quantity" => $product['cart_quantity'],
                    "totalAmount" => round(round($product['total_wt'], 2) * 100, 2),
                    "taxAmount" => round(round(($product['total_wt'] - $product['total']), 2) * 100, 2),
                    "productSKU" => $product['id_product'],
                    "productDescription" => substr(trim(strip_tags($product['description_short'])), 0, 50)
                ];
                $lineItems[] = $productos;
            }

            $orderArray = array_merge(array("billTo" => $billTo), array("lineItems" => $lineItems));
            $lang = new Language((int) Configuration::get('PS_LANG_DEFAULT'));
            $link = $this->context->link->getModuleLink('prestashopgatewaypayphone', 'validation', array(), Configuration::get('PS_SSL_ENABLED'));


            // Temporary clientTransactionId to generate the order
            $timestamp = time();
            $date = date('YmdHis', $timestamp);
            $milliseconds = round(microtime(true) * 1000) % 1000;
            $dateTimeString = $date . $milliseconds;
            $reference = $this->getReferenceOrder();
            $this->context->cookie->payphone_reference = $reference;

            $data->amount = round($amount * 100, 2);
            $data->amountWithTax = round($amount_with_tax * 100, 2);
            $data->tax = round($tax * 100, 2);
            $data->amountWithoutTax = round($amount_without_tax * 100, 2);

            $data->clientTransactionId = $reference . "_" . $dateTimeString;
            $data->responseUrl = $link;
            $data->cancellationUrl = $link;
            $data->lang = $lang->iso_code;
            $data->currency = $currency->iso_code;
            $data->email = $emailCart;
            $data->order = $orderArray;
            $data->token = $tk;
            try {
                if (!empty(Configuration::get('PAYPHONE_STORE_PPB')))
                    $data->storeId = Configuration::get('PAYPHONE_STORE_PPB');
            } catch (Exception $e) {
                PrestaShopLogger::addLog("Payphone Store Id empty", 4);
            }

            // Validate if you already have a transaction record
            $sql = new DbQuery();
            $sql->select('*');
            $sql->from('payphone_ppb', 'pt');
            $sql->where('pt.client_user_id = ' . $cart->id);
            $sql->orderBy('pt.id DESC');
            $transactions = Db::getInstance()->executeS($sql);
            $clientNames = $datosFactura['firstname'] . ' ' . $datosFactura['lastname'];

            $prevData = $transactions[0];
            $items = json_encode($lineItems);

            if ($prevData) {
                $updateData = [
                    'client_transaction_id' => pSQL($data->clientTransactionId),
                    'reference' => pSQL($data->clientTransactionId),
                    'amount' => $data->amount,
                    'line_items' => pSQL($items),
                    'phone_number' => pSQL($datosFactura['phone']),
                    'shipping' => $shipping,
                    'sub_total' => $sub_total,
                    'currency' => $data->currency,
                    'client_names' => pSQL($clientNames),
                    'email' => pSQL($data->email),
                ];
                $whereClause = ' `client_user_id` = ' . $cart->id;
                Db::getInstance()->update('payphone_ppb', $updateData, $whereClause);

            } else {
                $write = array();
                $write['client_transaction_id'] = pSQL($data->clientTransactionId);
                $write['reference'] = pSQL($data->clientTransactionId);
                $write['client_user_id'] = pSQL($cart->id);
                $write['client_names'] = pSQL($clientNames);
                $write['amount'] = $data->amount;
                $write['shipping'] = $shipping;
                $write['sub_total'] = $sub_total;
                $write['email'] = pSQL($data->email);
                $write['line_items'] = pSQL($items);
                $write['phone_number'] = pSQL($datosFactura['phone']);
                $write['date_add'] = date("Y-m-d H:i:s");
                $write['currency'] = $data->currency;
                Db::getInstance()->insert('payphone_ppb', $write);
            }

        } catch (Exception $e) {
            PrestaShopLogger::addLog("Php error processing payment", 4);
        }

        $config = ConfigurationManagerPpb::Instance();
        $ppbCssJs = $config->ppbCssJs;
        $css_uri = $this->getPathUri() . 'views/css/ps-ppb.css';

        $this->context->smarty->assign([
            'action' => $this->context->link->getModuleLink($this->name, 'validation', [], true),
            'paymentBoxCss' => $ppbCssJs . '.css',
            'paymentBoxJs' => $ppbCssJs . '.js',
            'ppb' => $this->getPathUri() . 'views/js/ppb.js',
            'ppbData' => json_encode($data),
            'css_uri' => $css_uri,
        ]);

        return $this->context->smarty->fetch('module:prestashopgatewaypayphone/views/templates/hook/displayPaymentByPpb.tpl');
    }

    /**
     * This hook is used to display additional information on bottom of order confirmation page
     *
     * @param array $params
     *
     * @return string
     */
    public function hookDisplayPaymentReturn(array $params)
    {
        if (empty($params['order'])) {
            return '';
        }

        /** @var Order $order */
        $order = $params['order'];

        if (false === Validate::isLoadedObject($order) || $order->module !== $this->name) {
            return '';
        }

        $css_uri = $this->getPathUri() . 'views/css/ps-ppb.css';
        $cartId = $order->id_cart;
        $orderReference = $order->reference;
        $hook_display_option = 1;

        $data_to_context = $this->getPayphoneOrderData($cartId, $css_uri, $orderReference, $hook_display_option);
        $data_to_context = json_decode($data_to_context, true);

        $this->context->smarty->assign($data_to_context);

        return $this->context->smarty->fetch('module:prestashopgatewaypayphone/views/templates/hook/payphone-order-template.tpl');
    }


    /**
     * Get data to display
     *
     * @param array $params
     *
     * @return string
     */
    public function getPayphoneOrderData($cartId, $css_uri, $orderReference, $hook_display_option)
    {
        // Get client_transaction_id for confirm
        $sql = new DbQuery();
        $sql->select('*');
        $sql->from('payphone_ppb', 'pt');
        $sql->where('pt.client_user_id = ' . $cartId);
        $sql->orderBy('pt.id DESC');
        $transactions = Db::getInstance()->executeS($sql);

        if ($transactions) {
            $clientTransactionId = $transactions[0]['client_transaction_id'];
            $amount = $transactions[0]['amount'];
            $line_items = $transactions[0]['line_items'];
            $store_name = $transactions[0]['store_name'];
            $status_code = $transactions[0]['status_code'];
            $date_add = $transactions[0]['date_add'];
            $client_names = $transactions[0]['client_names'];
            $authorization_code = $transactions[0]['authorization_code'];
            $phone_number = $transactions[0]['phone_number'];
            $email = $transactions[0]['email'];
            $shipping = $transactions[0]['shipping'];
            $sub_total = $transactions[0]['sub_total'];
            $currency = $transactions[0]['currency'];
            $card_bran = $transactions[0]['card_bran'];
            $deferred_message = $transactions[0]['deferred_message'];
            $transaction_id = $transactions[0]['transaction_id'];
            $client_pay_names = $transactions[0]['client_pay_names'];
            $reference = $transactions[0]['reference'];
            $message = $transactions[0]['message'];
            $client_pay_phone_number = $transactions[0]['client_pay_phone_number'];
        }

        $cancel_message = $message ? $message : $this->l('Canceled');
        $status_label = ($status_code == 3) ? $this->l('Approved') : $cancel_message;
        $color = ($status_code == 3) ? '#27C24C' : '#FB4F38';
        $items = json_decode($line_items, true);
        $data_to_show = [
            'moduleName' => $this->name,
            'clientTransactionId' => $clientTransactionId,
            'amount' => $amount,
            'shipping' => $shipping,
            'sub_total' => $sub_total,
            'currency' => $currency,
            'items' => $items,
            'module_dir' => _MODULE_DIR_,
            'css' => $css_uri,
            'store_name' => $store_name,
            'order_reference' => $orderReference,
            'color' => $color,
            'date_add' => date("d-m-Y H:i", strtotime($date_add)),
            'client_names' => $client_names,
            'authorization_code' => $authorization_code,
            'phone_number' => $phone_number,
            'email' => $email,
            'status_label' => $status_label,
            'card_bran' => $card_bran,
            'deferred_message' => $deferred_message,
            'transaction_id' => $transaction_id,
            'client_pay_names' => $client_pay_names,
            'client_pay_phone_number' => $client_pay_phone_number,
            'reference' => $reference,
            'message' => $message,
            'hook_display_option' => $hook_display_option,
        ];
        return json_encode($data_to_show);

    }

    /**
     * Check if the current plugin has enabled the same currency as the shopping cart.
     * @param type $cart Datos del carrito de compra
     * @return boolean
     */
    public function checkCurrency($cart)
    {


        $currency_order = new Currency($cart->id_currency);
        $currencies_module = $this->getCurrency($cart->id_currency);

        if (is_array($currencies_module))
            foreach ($currencies_module as $currency_module)
                if ($currency_order->id == $currency_module['id_currency'])
                    return true;
        return false;
    }


    /**
     * Factory of PaymentOption for binary Payment
     *
     * @return PaymentOption
     */
    private function getBinaryPaymentOption()
    {
        $binaryOption = new PaymentOption();
        $binaryOption->setModuleName($this->name);
        $binaryOption->setCallToActionText($this->l('Pay with Payphone (Visa and Mastercard)'));
        $binaryOption->setAdditionalInformation($this->context->smarty->fetch('module:prestashopgatewaypayphone/views/templates/front/paymentOptionPpb.tpl'));
        $binaryOption->setBinary(true);

        return $binaryOption;
    }

    /**
     * Create custom OrderState used for payment
     *
     * @param string $configurationKey Configuration key used to store OrderState identifier
     * @param array $nameByLangIsoCode An array of name for all languages, default is en
     * @param string $color Color of the label
     * @param bool $isLogable consider the associated order as validated
     * @param bool $isPaid set the order as paid
     * @param bool $isInvoice allow a customer to download and view PDF versions of his/her invoices
     * @param bool $isShipped set the order as shipped
     * @param bool $isDelivery show delivery PDF
     * @param bool $isPdfDelivery attach delivery slip PDF to email
     * @param bool $isPdfInvoice attach invoice PDF to email
     * @param bool $isSendEmail send an email to the customer when his/her order status has changed
     * @param string $template Only letters, numbers and underscores are allowed. Email template for both .html and .txt
     * @param bool $isHidden hide this status in all customer orders
     * @param bool $isUnremovable Disallow delete action for this OrderState
     * @param bool $isDeleted Set OrderState deleted
     *
     * @return bool
     */
    private function createOrderState(
        $configurationKey,
        array $nameByLangIsoCode,
        $color,
        $isLogable = false,
        $isPaid = false,
        $isInvoice = false,
        $isShipped = false,
        $isDelivery = false,
        $isPdfDelivery = false,
        $isPdfInvoice = false,
        $isSendEmail = false,
        $template = '',
        $isHidden = false,
        $isUnremovable = true,
        $isDeleted = false
    ) {
        $tabNameByLangId = [];

        foreach ($nameByLangIsoCode as $langIsoCode => $name) {
            foreach (Language::getLanguages(false) as $language) {
                if (Tools::strtolower($language['iso_code']) === $langIsoCode) {
                    $tabNameByLangId[(int) $language['id_lang']] = $name;
                } elseif (isset($nameByLangIsoCode['en'])) {
                    $tabNameByLangId[(int) $language['id_lang']] = $nameByLangIsoCode['en'];
                }
            }
        }

        $orderState = new OrderState();
        $orderState->module_name = $this->name;
        $orderState->name = $tabNameByLangId;
        $orderState->color = $color;
        $orderState->logable = $isLogable;
        $orderState->paid = $isPaid;
        $orderState->invoice = $isInvoice;
        $orderState->shipped = $isShipped;
        $orderState->delivery = $isDelivery;
        $orderState->pdf_delivery = $isPdfDelivery;
        $orderState->pdf_invoice = $isPdfInvoice;
        $orderState->send_email = $isSendEmail;
        $orderState->hidden = $isHidden;
        $orderState->unremovable = $isUnremovable;
        $orderState->template = $template;
        $orderState->deleted = $isDeleted;
        $result = (bool) $orderState->add();

        if (false === $result) {
            $this->_errors[] = sprintf(
                'Failed to create OrderState %s',
                $configurationKey
            );

            return false;
        }

        $result = (bool) Configuration::updateGlobalValue($configurationKey, (int) $orderState->id);

        if (false === $result) {
            $this->_errors[] = sprintf(
                'Failed to save OrderState %s to Configuration',
                $configurationKey
            );

            return false;
        }

        $orderStateImgPath = $this->getLocalPath() . 'views/images/logo.png';

        if (false === (bool) Tools::file_exists_cache($orderStateImgPath)) {
            $this->_errors[] = sprintf(
                'Failed to find icon file of OrderState %s',
                $configurationKey
            );

            return false;
        }

        if (false === (bool) Tools::copy($orderStateImgPath, _PS_ORDER_STATE_IMG_DIR_ . $orderState->id . '.gif')) {
            $this->_errors[] = sprintf(
                'Failed to copy icon of OrderState %s',
                $configurationKey
            );

            return false;
        }

        return true;
    }

    /**
     * Delete custom OrderState used for payment
     * We mark them as deleted to not break passed Orders
     *
     * @return bool
     */
    private function deleteOrderState()
    {
        $result = true;

        $orderStateCollection = new PrestaShopCollection('OrderState');
        $orderStateCollection->where('module_name', '=', $this->name);
        /** @var OrderState[] $orderStates */
        $orderStates = $orderStateCollection->getAll();

        foreach ($orderStates as $orderState) {
            $orderState->deleted = true;
            $result = $result && (bool) $orderState->save();
        }

        return $result;
    }

    /**
     * Install Tabs
     *
     * @return bool
     */
    public function installTabs()
    {
        if (Tab::getIdFromClassName(static::MODULE_ADMIN_CONTROLLER)) {
            return true;
        }

        $tab = new Tab();
        $tab->class_name = static::MODULE_ADMIN_CONTROLLER;
        $tab->module = $this->name;
        $tab->active = true;
        $tab->id_parent = -1;
        $tab->name = array();
        foreach (Language::getLanguages(true) as $lang) {
            $tab->name[$lang['id_lang']] = 'Controlador Admin para comprobar el pago con cajita de pagos';
        }

        return (bool) $tab->add();
    }

    /**
     * Uninstall Tabs
     *
     * @return bool
     */
    public function uninstallTabs()
    {
        $id_tab = (int) Tab::getIdFromClassName(static::MODULE_ADMIN_CONTROLLER);

        if ($id_tab) {
            $tab = new Tab($id_tab);

            return (bool) $tab->delete();
        }

        return true;
    }



    /**
     * Creates a table in the DB to record payment data
     * @return type
     */
    private function _installDb()
    {

        return Db::getInstance()->Execute('
                CREATE TABLE IF NOT EXISTS `' . _DB_PREFIX_ . 'payphone_ppb` (
                `id` INT(11) unsigned NOT NULL auto_increment,
                `transaction_id` INT(11) unsigned NULL,
                `message` VARCHAR(500) NULL,
                `email` VARCHAR(350) NULL,
                `store_name` VARCHAR(350) NULL,
                `client_names` VARCHAR(350) NULL,
                `client_pay_names` VARCHAR(350) NULL,
                `client_pay_phone_number` VARCHAR(20) NULL,
                `currency` VARCHAR(50) NULL,
                `phone_number` VARCHAR(20) NULL,
                `authorization_code` VARCHAR(100) NULL,
                `status_code` INT(1) NULL,
                `client_transaction_id` VARCHAR(50) NOT NULL,
                `client_user_id` VARCHAR(50) NULL,
                `deferred` TINYINT(1) NULL,
                `deferred_message` VARCHAR(255) NULL,
                `amount` VARCHAR(10) NULL,
                `sub_total` VARCHAR(10) NULL,
                `shipping` VARCHAR(10) NULL,
                `line_items` LONGTEXT NULL,
                `card_bran` VARCHAR(30) NULL,
                `date_add` DATETIME NOT NULL,
                `reference` VARCHAR(500) NULL,
                PRIMARY KEY (`id`)) ENGINE=' . _MYSQL_ENGINE_ . ' DEFAULT CHARSET=utf8 AUTO_INCREMENT=1');
    }

    /**
     * Executes the creation of order statuses for the payment button
     */
    private function _createOrderStates()
    {
        $this->_createPayphonePaymentStatus($this->payphone_pending, '#03a3e3', '', false, false, '', false);
        $this->_createPayphonePaymentStatus($this->payphone_green, '#4dbe50', 'payment', true, true, true, true);
        $this->_createPayphonePaymentStatus($this->payphone_red, '#fb4f38', 'payment_error', false, true, false, true);
        return true;
    }
    /**
     * Creates a purchase order status
     * @param type $array Array with order name in several languages
     * @param type $color Color that will display the order
     * @param type $template Template for mail sending
     * @param type $invoice Determines if the status generates invoice
     * @param type $send_email Determines if mail is sent
     * @param type $logable Determines if it is written to the log 
     */

    private function _createPayphonePaymentStatus($array, $color, $template, $invoice, $send_email, $paid, $logable)
    {
        foreach ($array as $key => $value) {
            $ow_status = Configuration::get($key);
            if ($ow_status === false) {
                $order_state = new OrderState();
            } else {
                $order_state = new OrderState((int) $ow_status);
            }
            $langs = Language::getLanguages();

            foreach ($langs as $lang) {
                $name = $value['en'];
                if ($lang['language_code'] == 'es') {
                    $name = $value['es'];
                }
                $name_clean = utf8_encode(html_entity_decode($name));
                $order_state->name[$lang['id_lang']] = $name_clean;
            }

            $order_state->invoice = $invoice;
            $order_state->send_email = $send_email;

            if ($template != '')
                $order_state->template = $template;

            if ($paid != '')
                $order_state->paid = $paid;

            $order_state->logable = $logable;
            $order_state->color = $color;
            $order_state->save();

            Configuration::updateValue($key, (int) $order_state->id);

            Tools::copy(dirname(__FILE__) . '/views/images/' . $key . '.gif', _PS_ROOT_DIR_ . '/img/os/' . (int) $order_state->id . '.gif');
        }
    }

    /**
     * Get a unique reference generated for the order
     * @return string
     */
    private function getReferenceOrder(): string
    {
        do {
            $reference = Order::generateReference();
        } while (Order::getByReference($reference)->count());
        return $reference;
    }

}