window.addEventListener('DOMContentLoaded', () => {
  let ppbData = document.getElementById('module-script-ppb').dataset.json;
  ppbData = JSON.parse(ppbData);
  let isAsyncResponse = true;
  let ppb = null;
  const scriptElement = document.getElementById('module-script-ppb');
  const actionUrl = scriptElement.dataset.action;
  const btnProcessAsync = document.getElementById('btnProcessAsync');

  function callAction() {
    const element = document.getElementById('pp-button');
    element.classList.add('ps-pbb-disabled');
    window.location.href = actionUrl;
  }

  function createButtonBox() {
    ppb = new PPaymentButtonBox({
      token: ppbData.token,
      lang: ppbData.lang ?? null,
      currency: ppbData.currency ?? null,
      clientTransactionId: ppbData.clientTransactionId,
      amount: ppbData.amount ?? null,
      amountWithoutTax: ppbData.amountWithoutTax ?? null,
      amountWithTax: ppbData.amountWithTax ?? null,
      email: ppbData.email ?? null,
      showMainButton: false,
      showPaymentMethodSelector: false,
      tax: ppbData.tax ?? null,
      service: ppbData.service ?? null,
      tip: ppbData.tip ?? null,
      transactionSummaryUrl: ppbData.responseUrl,
      order: ppbData.order ?? null,
      storeId: ppbData.storeId ?? null,
      customButtonId: 'btnProcessAsync',
      reference: ppbData.clientTransactionId,
      isAsyncResponse,
    }).render('pp-button');
  }

  function showSpinner() {
    const spinner = document.createElement('span');
    spinner.className = 'ppb-spinner-grow ppb-spinner-grow-sm ppb-me-2 ';
    spinner.setAttribute('role', 'status');
    spinner.setAttribute('aria-hidden', 'true');
    spinner.style.marginRight = '15px';
    btnProcessAsync.insertBefore(spinner, btnProcessAsync.firstChild);
  }

  function hideSpinner() {
    btnProcessAsync.removeChild(btnProcessAsync.firstChild);
  }

  createButtonBox();
  btnProcessAsync.disabled = !isAsyncResponse;
  btnProcessAsync.addEventListener('click', async () => {
    showSpinner();
    btnProcessAsync.disabled = true;
    const isInitialized = ppb.getIsInitialized();
    if (isInitialized) {
      try {
        const result = await ppb.startProcessPaymentAsync();
        if ((result.transactionId && result.clientTransactionId) || result.message || result.responseUrl) {
          btnProcessAsync.disabled = true;
          callAction();
        } else {
          btnProcessAsync.disabled = false;
          hideSpinner();
        }
      } catch (error) {
        btnProcessAsync.disabled = false;
        hideSpinner();
      }
    } else {
      btnProcessAsync.disabled = false;
      hideSpinner();
    }
  });

  const btnCard = document.getElementById('btnCard');
  const btnPayphone = document.getElementById('btnPayphone');
  btnPayphone.addEventListener('click', function () {
    ppb.changePaymentMethod('payphone');
    btnPayphone.classList.add('select-btn');
    btnCard.classList.remove('select-btn');
  });
  btnCard.addEventListener('click', function () {
    ppb.changePaymentMethod('card');
    btnCard.classList.add('select-btn');
    btnPayphone.classList.remove('select-btn');
  });
});
